#include <windows.h>
#include <wchar.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <assert.h>
#include <float.h>
#include <string.h>
#include <time.h>
#include"resource.h"
#define BUTTON_SIZE 43
#define BUTTON_MARGIN 2
#define BUTTONS_PER_ROW 11
#define WINDOW_WIDTH 510
#define WINDOW_HEIGHT 350 // Adjusted height to accommodate the additional row and edit control

// Additional button IDs
#define ID_SPACE 999
#define ID_BACKSPACE 1000
#define ID_ENTER 1001
#define IDC_EDIT 1002 // Added declaration for IDC_EDIT

HWND hEdit; // Edit control handle
wchar_t windowTitle[] = L"T Board";

wchar_t banglaAdditional[] = {
    L'\u09BE', L'\u09BF', L'\u09C0', L'\u09C1', L'\u09C2', L'\u09C3', L'\u09C7', L'\u09C8', L'\u09CB', L'\u09CC'
};

wchar_t banglaUnicode[] = {
    L'\u0985', L'\u0986', L'\u0987', L'\u0988', L'\u0989', L'\u098A', L'\u098B', L'\u098F', L'\u0990',
    L'\u0993', L'\u0994', L'\u0995', L'\u0996', L'\u0997', L'\u0998', L'\u0999', L'\u099A', L'\u099B',
    L'\u099C', L'\u099D', L'\u099E', L'\u099F', L'\u09A0', L'\u09A1', L'\u09A2', L'\u09A3', L'\u09A4',
    L'\u09A5', L'\u09A6', L'\u09A7', L'\u09A8', L'\u09AA', L'\u09AB', L'\u09AC', L'\u09AD', L'\u09AE',
    L'\u09AF', L'\u09B0', L'\u09B2', L'\u09B6', L'\u09B7', L'\u09B8', L'\u09B9', L'\u09DF', L'\u09CE', L'\u0982', L'\u0983', L'\u0981',L'\u09CD'
};

wchar_t banglaNumbers[] = {
    L'\u09E6', L'\u09E7', L'\u09E8', L'\u09E9', L'\u09EA', L'\u09EB', L'\u09EC', L'\u09ED', L'\u09EE', L'\u09EF'
};

void SendUnicodeCharacter(wchar_t unicodeChar) {
    INPUT ip;
    ip.type = INPUT_KEYBOARD;
    ip.ki.time = 0;
    ip.ki.dwExtraInfo = 0;
    ip.ki.dwFlags = KEYEVENTF_UNICODE;
    ip.ki.wVk = 0;
    ip.ki.wScan = unicodeChar;
    SendInput(1, &ip, sizeof(INPUT));
    ip.ki.dwFlags = KEYEVENTF_UNICODE | KEYEVENTF_KEYUP;
    SendInput(1, &ip, sizeof(INPUT));
    kbd(54);
}
void CreateUnicodeButton(HWND hwnd, int buttonID, wchar_t unicodeChar, int x, int y) {
    wchar_t buttonText[2] = {unicodeChar, L'\0'};
    CreateWindowExW(WS_EX_APPWINDOW | WS_EX_WINDOWEDGE | WS_EX_NOACTIVATE, L"Button", buttonText,
                    WS_VISIBLE | WS_CHILD | BS_PUSHBUTTON | BS_MULTILINE,
                    x, y, BUTTON_SIZE, BUTTON_SIZE, hwnd, (HMENU)buttonID, NULL, NULL);
}

LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch (uMsg) {
        case WM_CREATE: {
            int buttonID = 1;
            int x = BUTTON_MARGIN;
            int y = BUTTON_MARGIN;

            // Create buttons for the original array
            for (int i = 0; i < sizeof(banglaUnicode) / sizeof(banglaUnicode[0]); i++) {
                CreateUnicodeButton(hwnd, buttonID, banglaUnicode[i], x, y);
                if (banglaUnicode[i] == L'\u09CD') {
                    // Create Spacebar button after L'\u0981'
                   x += BUTTON_SIZE + BUTTON_MARGIN;
                    CreateUnicodeButton(hwnd, ID_SPACE, L'\u0020', x, y);
                    x += BUTTON_SIZE + BUTTON_MARGIN;

                    // Create Backspace button
                    CreateUnicodeButton(hwnd, ID_BACKSPACE, L'\b', x, y);
                    x += BUTTON_SIZE + BUTTON_MARGIN;

                    // Create Enter button
                    CreateUnicodeButton(hwnd, ID_ENTER, L'\r', x, y);
                }
                x += BUTTON_SIZE + BUTTON_MARGIN;
                buttonID++;
                if (buttonID % BUTTONS_PER_ROW == 1) {
                    x = BUTTON_MARGIN;
                    y += BUTTON_SIZE + BUTTON_MARGIN;
                }
            }

            // Create buttons for the additional array in a new row
            y += BUTTON_SIZE + BUTTON_MARGIN;
            x = BUTTON_MARGIN;
            for (int i = 0; i < sizeof(banglaAdditional) / sizeof(banglaAdditional[0]); i++) {
                CreateUnicodeButton(hwnd, buttonID, banglaAdditional[i], x, y);
                x += BUTTON_SIZE + BUTTON_MARGIN;
                buttonID++;
            }

            // Create buttons for the numbers in a new row
            y += BUTTON_SIZE + BUTTON_MARGIN;
            x = BUTTON_MARGIN;
            for (int i = 0; i < sizeof(banglaNumbers) / sizeof(banglaNumbers[0]); i++) {
                CreateUnicodeButton(hwnd, buttonID, banglaNumbers[i], x, y);
                x += BUTTON_SIZE + BUTTON_MARGIN;
                buttonID++;
            }

            // Create Edit control for text input
            hEdit = CreateWindowW(L"EDIT", L"",
                WS_BORDER | ES_LEFT | ES_MULTILINE | ES_AUTOVSCROLL,
                BUTTON_MARGIN, y + BUTTON_SIZE + BUTTON_MARGIN, WINDOW_WIDTH - 2 * BUTTON_MARGIN, BUTTON_SIZE,
                hwnd, (HMENU)IDC_EDIT, GetModuleHandle(NULL), NULL);

            break;
        }
        case WM_COMMAND: {
            int buttonID = LOWORD(wParam);
            if (buttonID >= 1 && buttonID <= sizeof(banglaUnicode) / sizeof(banglaUnicode[0])) {
                SendUnicodeCharacter(banglaUnicode[buttonID - 1]);
            } else if (buttonID == ID_SPACE || buttonID == ID_BACKSPACE || buttonID == ID_ENTER) {
                // Handle Spacebar, Backspace, and Enter button clicks
                switch (buttonID) {
                    case ID_SPACE:
                        SendUnicodeCharacter(L'\u0020'); // Spacebar
                        break;
                    case ID_BACKSPACE:
                        keybd_event(VK_BACK, 0, 0, 0);
                                    keybd_event(VK_BACK, 0, KEYEVENTF_KEYUP, 0);
                        break;
                    case ID_ENTER:
                        SendUnicodeCharacter(L'\u000D');
                        break;
                }
            } else if (buttonID > sizeof(banglaUnicode) / sizeof(banglaUnicode[0]) && buttonID <= (sizeof(banglaUnicode) / sizeof(banglaUnicode[0])) + sizeof(banglaAdditional) / sizeof(banglaAdditional[0])) {
                SendUnicodeCharacter(banglaAdditional[buttonID - 1 - sizeof(banglaUnicode) / sizeof(banglaUnicode[0])]);
            } else if (buttonID > (sizeof(banglaUnicode) / sizeof(banglaUnicode[0])) + sizeof(banglaAdditional) / sizeof(banglaAdditional[0])) {
                SendUnicodeCharacter(banglaNumbers[buttonID - 1 - sizeof(banglaUnicode) / sizeof(banglaUnicode[0]) - sizeof(banglaAdditional) / sizeof(banglaAdditional[0])]);
            }
            break;
        }
        case WM_SYSCOMMAND: {
            if ((wParam & 0xFFF0) == SC_MINIMIZE) {
                ShowWindow(hwnd, SW_MINIMIZE);
            } else if ((wParam & 0xFFF0) == SC_RESTORE) {
                ShowWindow(hwnd, SW_RESTORE);
            } else {
                return DefWindowProcW(hwnd, uMsg, wParam, lParam);
            }
            break;
        }
        case WM_GETMINMAXINFO: {
            // Prevent resizing and maximizing
            MINMAXINFO* lpMMI = (MINMAXINFO*)lParam;
            lpMMI->ptMinTrackSize.x = WINDOW_WIDTH;
            lpMMI->ptMinTrackSize.y = WINDOW_HEIGHT;
            lpMMI->ptMaxTrackSize.x = WINDOW_WIDTH;
            lpMMI->ptMaxTrackSize.y = WINDOW_HEIGHT;
            break;
        }
        case WM_CLOSE: {
            DestroyWindow(hwnd);
            break;
        }
        case WM_DESTROY: {
            PostQuitMessage(0);
            break;
        }
        case WM_ACTIVATE: {
            if (LOWORD(wParam) == WA_INACTIVE) {
                // Window is losing focus, bring it to the front
                SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
            }
            break;
        }
        case WM_SETCURSOR: {
            // Change the cursor when it enters the client area of the window
            SetCursor(LoadCursor(NULL, IDC_HAND));
            return TRUE;
        }
        default: {
            return DefWindowProcW(hwnd, uMsg, wParam, lParam);
        }
    }
    return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    WNDCLASSW wc = {};
    wc.lpfnWndProc = WindowProc;
    wc.hInstance = hInstance;
    wc.lpszClassName = L"BanglaKeyboardClass";
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    //ICON hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MYICON));
wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MYICON));
// Set
    RegisterClassW(&wc);

    HWND hwnd = CreateWindowExW(WS_EX_APPWINDOW | WS_EX_WINDOWEDGE | WS_EX_NOACTIVATE, L"BanglaKeyboardClass", windowTitle,
                                WS_OVERLAPPEDWINDOW & ~WS_THICKFRAME & ~WS_MAXIMIZEBOX, CW_USEDEFAULT, CW_USEDEFAULT, WINDOW_WIDTH, WINDOW_HEIGHT, NULL, NULL, hInstance, NULL);

    ShowWindow(hwnd, SW_SHOWNORMAL);
    UpdateWindow(hwnd);

    SetForegroundWindow(GetShellWindow());

    MSG msg;
    while (GetMessageW(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessageW(&msg);
    }
    return 0;
}
